const express = require('express');
const app = express();
require('dotenv').config();
const cors = require('cors');

// Importing route modules
const signup = require('./routes/signup');
const verify = require('./routes/emailVerify');
const login = require('./routes/login');
const stripe = require('./routes/stripePayment');

// Set the port from environment or fallback to 8080
const PORT = process.env.PORT || 8080;

// Azure Website URL from environment variable or default value
const AZURE_WEBSITE_URL = process.env.AZURE_WEBSITE_URL || 'https://www.alegriaquest.com';

app.use(express.json());
app.use(cors({
    origin: [AZURE_WEBSITE_URL, 'http://localhost:4200'],
    methods: ['GET', 'DELETE', 'PUT', 'POST']
}));

// Route middleware
app.use('/signup', signup);
app.use('/verify', verify);
app.use('/login', login);
app.use('/stripe', stripe);

// Base route for health check or default message
app.get('/', (req, res) => {
    res.send('success');
});

// Test routes for login and signup (for development)
app.post('/logintest', (req, res) => {
    const user = req.body;
    res.send(user);
});

app.post('/signuptest', (req, res) => {
    const user = req.body;
    res.send(user);
});

// Start the server
app.listen(PORT, () => {
    console.log(`Server is listening on port ${PORT}`);
});
